@@ -1,7 +1,8 @@
 import asyncio
-import click
 import json
 import sys
+
+import click
 import yaml
 
 from distributed.deploy.spec import run_spec
@@ -31,11 +32,11 @@ def main(args, spec: str, spec_file: str):
     async def run():
         servers = await run_spec(_spec, *args)
         try:
-            await asyncio.gather(*[w.finished() for w in servers.values()])
+            await asyncio.gather(*(w.finished() for w in servers.values()))
         except KeyboardInterrupt:
-            await asyncio.gather(*[w.close() for w in servers.values()])
+            await asyncio.gather(*(w.close() for w in servers.values()))
 
-    asyncio.get_event_loop().run_until_complete(run())
+    asyncio.run(run())
 
 
 if __name__ == "__main__":
