@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import os
 import socket
@@ -5,9 +7,14 @@ import subprocess
 import sys
 import uuid
 import zipfile
+from collections.abc import Awaitable
+from typing import TYPE_CHECKING
 
 from dask.utils import funcname, tmpfile
 
+if TYPE_CHECKING:
+    from distributed.scheduler import Scheduler  # circular import
+
 logger = logging.getLogger(__name__)
 
 
@@ -25,8 +32,11 @@ class SchedulerPlugin:
     Plugins are often used for diagnostics and measurement, but have full
     access to the scheduler and could in principle affect core scheduling.
 
-    To implement a plugin implement some of the methods of this class and add
-    the plugin to the scheduler with ``Scheduler.add_plugin(myplugin)``.
+    To implement a plugin:
+
+    1. subclass this class
+    2. override some of its methods
+    3. add the plugin to the scheduler with ``Scheduler.add_plugin(myplugin)``.
 
     Examples
     --------
@@ -45,28 +55,31 @@ class SchedulerPlugin:
     >>> scheduler.add_plugin(plugin)  # doctest: +SKIP
     """
 
-    async def start(self, scheduler):
+    async def start(self, scheduler: Scheduler) -> None:
         """Run when the scheduler starts up
 
         This runs at the end of the Scheduler startup process
         """
-        pass
 
-    async def close(self):
+    async def before_close(self) -> None:
+        """Runs prior to any Scheduler shutdown logic"""
+
+    async def close(self) -> None:
         """Run when the scheduler closes down
 
         This runs at the beginning of the Scheduler shutdown process, but after
         workers have been asked to shut down gracefully
         """
-        pass
 
-    def update_graph(self, scheduler, dsk=None, keys=None, restrictions=None, **kwargs):
+    def update_graph(
+        self, scheduler: Scheduler, keys: set[str], restrictions: dict, **kwargs
+    ) -> None:
         """Run when a new graph / tasks enter the scheduler"""
 
-    def restart(self, scheduler, **kwargs):
+    def restart(self, scheduler: Scheduler) -> None:
         """Run when the scheduler restarts itself"""
 
-    def transition(self, key, start, finish, *args, **kwargs):
+    def transition(self, key: str, start: str, finish: str, *args, **kwargs) -> None:
         """Run whenever a task changes state
 
         Parameters
@@ -81,16 +94,18 @@ class SchedulerPlugin:
             This may include worker ID, compute time, etc.
         """
 
-    def add_worker(self, scheduler=None, worker=None, **kwargs):
+    def add_worker(self, scheduler: Scheduler, worker: str) -> None | Awaitable[None]:
         """Run when a new worker enters the cluster"""
 
-    def remove_worker(self, scheduler=None, worker=None, **kwargs):
+    def remove_worker(
+        self, scheduler: Scheduler, worker: str
+    ) -> None | Awaitable[None]:
         """Run when a worker leaves the cluster"""
 
-    def add_client(self, scheduler=None, client=None, **kwargs):
+    def add_client(self, scheduler: Scheduler, client: str) -> None:
         """Run when a new client connects"""
 
-    def remove_client(self, scheduler=None, client=None, **kwargs):
+    def remove_client(self, scheduler: Scheduler, client: str) -> None:
         """Run when a client disconnects"""
 
 
@@ -127,7 +142,8 @@ class WorkerPlugin:
     ...                 exc_info=exc_info
     ...             )
 
-    >>> plugin = ErrorLogger()
+    >>> import logging
+    >>> plugin = ErrorLogger(logging)
     >>> client.register_worker_plugin(plugin)  # doctest: +SKIP
     """
 
@@ -161,24 +177,6 @@ class WorkerPlugin:
         kwargs : More options passed when transitioning
         """
 
-    def release_key(self, key, state, cause, reason, report):
-        """
-        Called when the worker releases a task.
-
-        Parameters
-        ----------
-        key : string
-        state : string
-            State of the released task.
-            One of waiting, ready, executing, long-running, memory, error.
-        cause : string or None
-            Additional information on what triggered the release of the task.
-        reason : None
-            Not used.
-        report : bool
-            Whether the worker should report the released task to the scheduler.
-        """
-
 
 class NannyPlugin:
     """Interface to extend the Nanny
@@ -215,9 +213,12 @@ class NannyPlugin:
         """Run when the nanny to which the plugin is attached to is closed"""
 
 
-def _get_worker_plugin_name(plugin) -> str:
-    """Returns the worker plugin name. If plugin has no name attribute
-    a random name is used."""
+def _get_plugin_name(plugin) -> str:
+    """Return plugin name.
+
+    If plugin has no name attribute a random name is used.
+
+    """
     if hasattr(plugin, "name"):
         return plugin.name
     else:
@@ -270,7 +271,7 @@ class PipInstall(WorkerPlugin):
         self.pip_options = pip_options
 
     async def setup(self, worker):
-        from ..lock import Lock
+        from distributed.lock import Lock
 
         async with Lock(socket.gethostname()):  # don't clobber one installation
             logger.info("Pip installing the following packages: %s", self.packages)
@@ -393,7 +394,7 @@ class UploadDirectory(NannyPlugin):
                 self.data = f.read()
 
     async def setup(self, nanny):
-        fn = os.path.join(nanny.local_directory, f"tmp-{str(uuid.uuid4())}.zip")
+        fn = os.path.join(nanny.local_directory, f"tmp-{uuid.uuid4()}.zip")
         with open(fn, "wb") as f:
             f.write(self.data)
 
